@@ -469,6 +469,45 @@ perl_ARMessageStruct(ARControlStruct * ctrl, ARMessageStruct * in)
 }
 #endif
 
+ARCoordList *
+dup_ARCoordList(ARControlStruct * ctrl, ARCoordList * in) 
+{
+	if ( in ) {
+		ARCoordList *n = MALLOCNN(sizeof(ARCoordList));
+		if (!n) return NULL;
+		n->numItems = in->numItems;
+		if (n->numItems) {
+			AMALLOCNN(n->coords, n->numItems, ARCoordStruct);
+			memcpy(n->coords, in->coords, 
+			       sizeof(ARCoordStruct) * in->numItems);
+		}
+		return n;
+	}
+	return NULL;
+}
+
+ARByteList *
+dup_ARByteList(ARControlStruct * ctrl, ARByteList * in) 
+{
+	if ( in ) {
+		ARByteList *n = MALLOCNN(sizeof(ARByteList));
+
+		if (!n) return NULL;
+		n->numItems = in->numItems;
+		n->type = in->type;
+		if (in->numItems) {
+			n->bytes = MALLOCNN(in->numItems);
+			if (!n->bytes) {
+				AP_FREE(n);
+				return NULL;
+			}
+			memcpy(n->bytes, in->bytes, in->numItems);
+		}
+		return n;
+	}
+	return NULL;
+}
+
 #if AR_EXPORT_VERSION >= 7L
 SV             *
 perl_AREnumItemStruct(ARControlStruct * ctrl, AREnumItemStruct * in)
@@ -506,6 +545,96 @@ perl_AREnumQueryStruct(ARControlStruct * ctrl, AREnumQueryStruct * in)
 	return newRV_noinc((SV *) hash);
 }
 
+#if AR_EXPORT_VERSION >= 7
+void
+dup_ARFuncCurrencyList(ARFuncCurrencyList *dst, ARFuncCurrencyList *src)
+{
+	if( dst && src ) {
+		dst->numItems = src->numItems;
+		AMALLOCNN(dst->funcCurrencyList,
+			  src->numItems,
+			  ARFuncCurrencyStruct);
+
+		memcpy(dst->funcCurrencyList, 
+		       src->funcCurrencyList,
+		       sizeof(ARFuncCurrencyStruct) * src->numItems);
+	}
+}
+
+ARCurrencyStruct *
+dup_ARCurrencyStruct(ARControlStruct * ctrl, ARCurrencyStruct * in)
+{
+	if ( in && in->value ) {
+		ARCurrencyStruct *n = MALLOCNN(sizeof(ARCurrencyStruct));
+		if ( !n ) return NULL;
+		if (in->value)
+			strcpy(n->value, in->value);
+		n->conversionDate = in->conversionDate;
+		strncpy(n->currencyCode, in->currencyCode, 
+		       sizeof(ARCurrencyCodeType));
+		dup_ARFuncCurrencyList(&(n->funcList), &(in->funcList));
+	}
+	return NULL;
+}
+
+SV             *
+perl_ARFuncCurrencyStruct(ARControlStruct * ctrl, ARFuncCurrencyStruct * in)
+{
+	HV            *hash = newHV();
+
+	if(in->value) {
+		hv_store(hash, "value", strlen("value"),
+			 newSVpv(in->value, 0), 0);
+	} else {
+		hv_store(hash, "value", strlen("value"),
+			 &PL_sv_undef, 0);
+	}
+
+	if(in->currencyCode) {
+		hv_store(hash, "currencyCode", strlen("currencyCode"),
+			 newSVpv(in->currencyCode, 0), 0);
+	} else {
+		hv_store(hash, "currencyCode", strlen("currencyCode"),
+			 &PL_sv_undef, 0);
+	}
+
+	return newRV_noinc((SV *) hash);
+}
+
+
+SV             *
+perl_ARCurrencyStruct(ARControlStruct * ctrl, ARCurrencyStruct * in)
+{
+	HV            *hash = newHV();
+
+	if(in->value) {
+		hv_store(hash, "value", strlen("value"),
+			 newSVpv(in->value, 0), 0);
+	} else {
+		hv_store(hash, "value", strlen("value"),
+			 &PL_sv_undef, 0);
+	}
+
+	if(in->currencyCode) {
+		hv_store(hash, "currencyCode", strlen("currencyCode"),
+			 newSVpv(in->currencyCode, 0), 0);
+	} else {
+		hv_store(hash, "currencyCode", strlen("currencyCode"),
+			 &PL_sv_undef, 0);
+	}
+
+	hv_store(hash, "conversionDate", strlen("conversionDate"),
+		 newSViv(in->conversionDate), 0);
+
+	hv_store(hash, "funcList", strlen("funcList"),
+		 perl_ARList(ctrl, (ARList *)&(in->funcList),
+			     (ARS_fn) perl_ARFuncCurrencyStruct,
+			     sizeof(ARFuncCurrencyStruct)), 0);
+
+	return newRV_noinc((SV *) hash);
+}
+#endif
+
 SV             *
 perl_AREnumLimitsStruct(ARControlStruct * ctrl, AREnumLimitsStruct * in)
 {
@@ -1136,12 +1265,20 @@ perl_ARActiveLinkActionStruct(ARControlStruct * ctrl, ARActiveLinkActionStruct *
 		       perl_ARActiveLinkMacroStruct(ctrl, &in->u.macro), 0);
 		break;
 	case AR_ACTIVE_LINK_ACTION_FIELDS:
-		hv_store(hash,  "assign_fields", strlen("assign_fields") ,
-			 perl_ARList(ctrl,
-				     (ARList *) & in->u.fieldList,
-				     (ARS_fn) perl_ARFieldAssignStruct,
-				     sizeof(ARFieldAssignStruct)), 0);
-		break;
+          {
+            ARList *fieldList = NULL;
+#if AR_EXPORT_VERSION >= 8L
+            fieldList = (ARList *) & in->u.setFields;
+#else
+            fieldList = (ARList *) & in->u.fieldList;
+#endif
+            hv_store(hash,  "assign_fields", strlen("assign_fields") ,
+                     perl_ARList(ctrl,
+                                 fieldList,
+                                 (ARS_fn) perl_ARFieldAssignStruct,
+                                 sizeof(ARFieldAssignStruct)), 0);
+          }
+          break;
 	case AR_ACTIVE_LINK_ACTION_PROCESS:
 		hv_store(hash,  "process", strlen("process") , newSVpv(in->u.process, 0), 0);
 		break;
@@ -1164,13 +1301,21 @@ perl_ARActiveLinkActionStruct(ARControlStruct * ctrl, ARActiveLinkActionStruct *
 		break;
 #if AR_EXPORT_VERSION >= 4
         case AR_ACTIVE_LINK_ACTION_FIELDP:
-		/*ARPushFieldsList;*/
-		hv_store(hash,  "fieldp", strlen("fieldp") ,
-			 perl_ARList(ctrl, 
-				     (ARList *)& in->u.pushFieldsList,
-				     (ARS_fn) perl_ARPushFieldsStruct,
-				     sizeof(ARPushFieldsStruct)), 0);
-		break;
+          {
+            ARList *pushFields = NULL;
+#if AR_EXPORT_VERSION >= 8L
+            pushFields = (ARList *)& in->u.pushFields;
+#else
+            pushFields = (ARList *)& in->u.pushFieldsList;
+#endif
+            /*ARPushFieldsList;*/
+            hv_store(hash,  "fieldp", strlen("fieldp") ,
+                     perl_ARList(ctrl, 
+                                 pushFields,
+                                 (ARS_fn) perl_ARPushFieldsStruct,
+                                 sizeof(ARPushFieldsStruct)), 0);
+          }
+          break;
         case AR_ACTIVE_LINK_ACTION_SQL:
 		/*ARSQLStruct;*/
 		hv_store(hash,  "sqlCommand", strlen("sqlCommand") ,
@@ -1272,26 +1417,42 @@ perl_ARFilterActionStruct(ARControlStruct * ctrl, ARFilterActionStruct * in)
 		hv_store(hash,  "log", strlen("log") , newSVpv(in->u.logFile, 0), 0);
 		break;
 	case AR_FILTER_ACTION_FIELDS:
-		hv_store(hash,  "assign_fields", strlen("assign_fields") ,
-			 perl_ARList(ctrl,
-				     (ARList *) & in->u.fieldList,
-				     (ARS_fn) perl_ARFieldAssignStruct,
-				     sizeof(ARFieldAssignStruct)), 0);
-		break;
+          {
+            ARList *setFields = NULL;
+#if AR_EXPORT_VERSION >= 8L
+            setFields = (ARList *) & in->u.setFields;
+#else
+            setFields = (ARList *) & in->u.fieldList;
+#endif
+            hv_store(hash,  "assign_fields", strlen("assign_fields") ,
+                     perl_ARList(ctrl,
+                                 setFields,
+                                 (ARS_fn) perl_ARFieldAssignStruct,
+                                 sizeof(ARFieldAssignStruct)), 0);
+          }
+          break;
 	case AR_FILTER_ACTION_PROCESS:
 		hv_store(hash,  "process", strlen("process") , newSVpv(in->u.process, 0), 0);
 		break;
 #if AR_EXPORT_VERSION >= 4
  /* added cases for new ACTIONS in ARS v4.0 API, Geoff Endresen, 6/28/2000
     copied from AR_ACTIVE_LINK_ACTION_FIELP */
-         case AR_FILTER_ACTION_FIELDP:
-                 /*ARPushFieldsList;*/
-                 hv_store(hash,  "fieldp", strlen("fieldp") ,
-                          perl_ARList(ctrl,
-                                      (ARList *)& in->u.pushFieldsList,
-                                      (ARS_fn) perl_ARPushFieldsStruct,
-                                      sizeof(ARPushFieldsStruct)),0);
-                 break;
+        case AR_FILTER_ACTION_FIELDP:
+          {
+            ARList *pushFields = NULL;
+#if AR_EXPORT_VERSION >= 8L
+            pushFields = (ARList *)& in->u.pushFields;
+#else
+            pushFields = (ARList *)& in->u.pushFieldsList;
+#endif
+            /*ARPushFieldsList;*/
+            hv_store(hash,  "fieldp", strlen("fieldp") ,
+                     perl_ARList(ctrl,
+                                 pushFields,
+                                 (ARS_fn) perl_ARPushFieldsStruct,
+                                 sizeof(ARPushFieldsStruct)),0);
+          }
+          break;
          case AR_FILTER_ACTION_SQL:
                  /*ARSQLStruct;*/
                  hv_store(hash,  "sqlCommand", strlen("sqlCommand") ,
@@ -1540,15 +1701,15 @@ perl_ARFieldLimitStruct(ARControlStruct * ctrl, ARFieldLimitStruct * in)
 			 0);
 #endif
 		return newRV_noinc((SV *) hash);
-
-	case AR_DATA_TYPE_BYTES:
-	case AR_DATA_TYPE_DECIMAL:
-	case AR_DATA_TYPE_ATTACH:
+		
 #if AR_EXPORT_VERSION >= 7
 	case AR_DATA_TYPE_CURRENCY:
 	case AR_DATA_TYPE_DATE:
 	case AR_DATA_TYPE_TIME_OF_DAY:
 #endif
+	case AR_DATA_TYPE_BYTES:
+	case AR_DATA_TYPE_DECIMAL:
+	case AR_DATA_TYPE_ATTACH:
 
 	case AR_DATA_TYPE_NULL:
 	default:
@@ -2113,14 +2274,33 @@ dup_Value(ARControlStruct * ctrl, ARValueStruct * n, ARValueStruct * in)
 	case AR_DATA_TYPE_TIME:
 	case AR_DATA_TYPE_BITMASK:
 	case AR_DATA_TYPE_ENUM:
+	case AR_DATA_TYPE_ULONG:
+#if AR_EXPORT_VERSION > 6L
+	case AR_DATA_TYPE_DATE:
 		n->u = in->u;
 		break;
+	case AR_DATA_TYPE_CURRENCY:
+		n->u.currencyVal = dup_ARCurrencyStruct(ctrl, 
+							in->u.currencyVal);
+		break;
+#endif
 	case AR_DATA_TYPE_CHAR:
 		n->u.charVal = strdup(in->u.charVal);
 		break;
+	case AR_DATA_TYPE_DECIMAL:
+		n->u.decimalVal = strdup(in->u.decimalVal);
+		break;
 	case AR_DATA_TYPE_DIARY:
 		n->u.diaryVal = strdup(in->u.diaryVal);
 		break;
+	case AR_DATA_TYPE_COORDS:
+		n->u.coordListVal = dup_ARCoordList(ctrl,
+						    in->u.coordListVal);
+		break;
+	case AR_DATA_TYPE_BYTES:
+		n->u.byteListVal = dup_ARByteList(ctrl,
+					       in->u.byteListVal);
+		break;
 	}
 }
 
@@ -2811,6 +2991,44 @@ sv_to_ARValue(ARControlStruct * ctrl, SV * in, unsigned int dataType,
 		case AR_DATA_TYPE_BITMASK:
 			out->u.maskVal = SvIV(in);
 			break;
+#if AR_EXPORT_VERSION >= 7L
+		case AR_DATA_TYPE_DATE:
+			out->u.dateVal = SvIV(in);
+			break;
+		case AR_DATA_TYPE_CURRENCY:
+#if 0 /* XX FINISH */
+			if (SvROK(in)) {
+				if (SvTYPE (hash = (HV *)SvRV(in)) == SVt_PVHV) {
+					fetch = hv_fetch(hash, "value", 5, FALSE);
+					if (!fetch) {
+						ARError_add(AR_RETURN_ERROR, AP_ERR_CURRENCY_STRUCT);
+						return -1;
+					}
+					val = *fetch;
+					if(!(SvOK(val) && SvTYPE(val) != SVt_RV)) {
+						ARError_add(AR_RETURN_ERROR, AP_ERR_CURRENCY_STRUCT);
+						return -1;
+					}
+
+					fetch = hv_fetch(hash, "currencyCode", 
+							 strlen("currencyCode"), FALSE);
+					if (!fetch) {
+						ARError_add(AR_RETURN_ERROR, AP_ERR_CURRENCY_STRUCT);
+						return -1;
+					}
+					type = *fetch; 
+					if(!(SvOK(val) && SvTYPE(val) != SVt_RV)) {
+						ARError_add(AR_RETURN_ERROR, AP_ERR_CURRENCY_STRUCT);
+						return -1;
+					}
+					out->u.currencyVal = MALLOCNN(sizeof(ARCurrencyStruct));
+					out->u.currencyVal->value = strdup(SvPV(val, PL_na));
+					out->u.currencyVal->currencyCode = strdup(SvPV(type, PL_na));
+#endif
+#endif /* 7L */
+
+					
+
 #if AR_EXPORT_VERSION >= 3
 		case AR_DATA_TYPE_BYTES:
 			if (SvROK(in)) {
